/*

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                Arlips compiler v. 3.1           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

File alex.c, April 2006 (lexical analyzer)

Copyright (C) 2006 Emilio Vivancos Rubio

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                              Contact                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Author:              Emilio Vivancos Rubio  vivancos@dsic.upv.es
Developer Arlips v2: Mario Gonzlez Gimnez mgonzale@dsic.upv.es
Developer Arlips v3: Carlos Garca Montoro  cargarcia@dsic.upv.es

Departamento de Sistemas Informticos y Computacin
Universidad Politcnica de Valencia
Camino de Vera s/n
46022 Valencia
Spain

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

*/


/****************************************************************************/
/************************* ANALIZADOR LEXICO ********************************/
/****************************************************************************/

#include "alex.h"
#include "es.h"
#include "tipos.h"
#include "tds.h"
//#include "arlips_kdm_parser/parser.h" //parser para convertir datos del kdm en estructuras Arlips
#include "errores.h"

int f;                  /* descriptor del fichero de entrada */
char ch;                /* caracter que se esta procesando */
char enteroseguidodospuntos;
char linea[long_linea];
char linea_orig[long_linea];   /* es la linea que obtenemos sin cambiar de minusculas */
                               /* a mayusculas: la linea tal como esta en el fuente */

//La siguiente variable no funciona adecuadamente con valor TRUE.
//Se ha modificado obtch para que botenga siempre la cadena tal cual,
//y se debe considerar las partes del lenguaje en las que no debe existir
//diferencias entre escribir minsculas o maysculas (bsicamente, las 
//palabras reservadas.
int obt_orig=FALSE;            /* este booleano si se activa devuelve la cadena del original */
                               /* (sin pasar a mayusculas) */


int numlinea,contcarac,longlineactual,aux,opes;
char regla_error;
char leidosigcar;
char errorahora;
char error_rango; /* para cuando el entero es mayor del rango permitido */
char tipdelreg;
char cterepetida; /* me indicara si hay una constante repetida */

/* pila con los descriptores de ficheros */
int pila_fich[max_pila_f];  /* esta pila es para mantener los descriptores de los ficheros */
                    /* que se van abriendo, cuando se encuentran directivas */
                    /* include para el analizador lexico */

//Pila con los nombres de los ficheros para mensajes de error.
extern char *pila_nombres_ficheros[max_pila_f];


int cima_pila;
char fich_aux[max_cadena_comillas+1];
int is_kdm_level; //Indica si se est procesando un nivel KDM

/* lista con los ficheros externos */
/* ( 25 como maximo ) */
extern char * fich_ext[max_f_ext];
extern int num_fich_ext;




/****** Valores devueltos por el analizador lexico ******/

char id[long_ident+1];  /* pongo uno mas para el caracter nulo */
char idaux[long_ident+1];
char cadena_comillas[max_cadena_comillas+1];       /* las cadenas para los printf */
char simbolo;  /* ultimo simbolo leido */
float num;       /* valor de una constante numerica */
char esreal;    /* me dice si he leido un numero real */
extern int artis_mode;
extern int nomain;

/*** array de palabras reservadas */

//POR UN MUNDO MEJOR: Cada fila debe tener 5 elementos
//(se empieza por el 0, como hace C por defecto)
//si se aaden nuevos smbolos, se tienen que meter en
//ORDEN ALFABTICO, y se debe revisar las filas para que tenga
//5 ELEMENTOS/FILA
char pr[] [long_ident]={" "           , "ACCESOR"     , "ADDHEAD"      , "ADDTAIL"     , "AND"        , // 0 - 4 
			"ARRAY"	      ,	"BOOLEAN"     , "CHAR"         , "CLASS"       , "DEFAULT"    , // 5 - 9 
			"DEFCLASS"    ,	"DEFCONST"    , "DEFFUNCTION"  , "DEFINSTANCES", "DEFPATTERN" , //10 - 14 
			"DEFRULE"     ,	"DEFVAR"      , "DEPTH"        , "END"         , "EXTERN"     , //15 - 19
			"EXTFILE"     ,	"FACTS"       , "FALSE"        , "FRECUENCY"   , "HALT"       , //20 - 24
			"HEAD"        ,	"IMPORT"      ,	"INCLUDE"      , "INPUT"       , "INTEGER"    , //25 - 29
			"INTLIST"     ,	"ISA"         ,	"KDM"          , "NBPREDICT"   , "NBVALUES"   , //30 - 34
			"NOT"         , "OF"          ,	"OR"           , "POPHEAD"     , "POPTAIL"    , //35 - 39
			"PRINT"       , "PROCEDURE"   ,	"PROGRAM"      , "READ"        , "REAL"       , //40 - 44
			"REALIST"     , "RECORD"      ,	"RETURN"       , "RULES"       , "SETSTRATEGY", //45 - 49
			"SIZE"        , "SLOT"        ,	"STATICSLOT"   , "TAIL"        , "TEMPOSLOT"  , //50 - 54
			"TRUE"        , "TYPE"        ,	"WIDTH"        , "WRITE"                        //55 - 58
};

//   /* pongo tamao 13, para introducir el nulo */

//num_pal_reser se encuentra definida en constant.h
//Recuerda modificarla si vas a aadir una nueva palabra reservada.
char simpalres[num_pal_reser+1]={-1,             accesorsim,     addheadsim,     addtailsim,     opysim,
				 arraysim,	 booleansim,     charsim,        clasesim,       defaultsim,
				 defclasesim,	 defconstsim,    deffunctionsim, definstancesim, defpatternsim,
				 defrulesim,	 defvariables,   depthsim,	 endsim,         externsim,
				 extfilesim,	 factssim,       falsesim,	 frecuencysim,   haltsim,
				 headsim,	 importsim,	 includesim,	 inputsim,	 intesim,        
				 intlistsim,	 isasim,	 kdmsim,         nbpredictsim,	 nbvaluessim,    
				 opnosim,	 ofsim,		 oposim,         popheadsim,	 poptailsim,     
				 prinsim,	 proceduresim,	 programsim,     readsim,	 realsim,        
				 realistsim,	 recordsim,	 returnsim,      rulesim,	 setstrategysim, 
				 sizesim,	 slotsim,	 slotsim,        tailsim,	 temposlotsim,   
				 truesim,	 typesim,	 widthsim,       writesim
};



char f_kdm[] = "kdm.arl"; /*Fichero que contendr el cdigo del kdm, en caso de que se importe.*/


char uni[25];   /* servir para la union de conjuntos */

int Inicializa_Analizador_Lexico(char * fich_ent) {
  /* Esta funcion se encarga de inicializar el analizador lexico */
  f=abrir_fichero_lectura(fich_ent);
  if (f!=-1) {
    cima_pila=0;
    pila_fich[0]=f;
    pila_nombres_ficheros[0] = (char *)malloc(strlen(fich_ent) + 1);
    strcpy(pila_nombres_ficheros[0], fich_ent);
    is_kdm_level = FALSE;
    contcarac=0; longlineactual=0; numlinea=0; ch=' ';
    errorahora=FALSE; leidosigcar=FALSE; enteroseguidodospuntos=FALSE;
    num_fich_ext= 0;
    return (FALSE);
  } 
  else 
    return(TRUE); /* se ha producido un error al abrir el fichero */
}

void Cierra_Analizador_Lexico() {
  /* esta funcion cierra todos los ficheros abiertos que se encuentran en la
     pila */
  while(cima_pila>=0) {
    f=pila_fich[cima_pila];
    if (close(f))
      fprintf(stderr,"ERROR: Error closing file");
    free(pila_nombres_ficheros[cima_pila]);
    cima_pila--;
    is_kdm_level = FALSE;
  }
}


char obtch()
{
  int i;
  int entrecomillado;
  
  if(ch=='$') menerror_1(99);
  if(contcarac==longlineactual)
    {
      contcarac=0; i=1; numlinea++;
      entrecomillado = FALSE;
      while(! eof(f) && ! eoln(f))
	{
	  /* f es el fichero abierto */
	  
	  if(i<long_linea)
	    {
	      if(read(f,&ch,1)!=1) printf("ERROR: Error reading file\n");
	      linea_orig[i]=ch;
	      //if (!entrecomillado) if((ch>=97)&&(ch<=122)) ch=ch-32; /* paso de minusculas a mayusculas */
	      if (ch=='"') entrecomillado=!entrecomillado;
	      linea[i]=ch;
	      i++;
	    }
	  else
	    {
	      menerror_1(53);
	      while(!eof(f) && !eoln(f)) read(f,&ch,1);
	    }
	}
      if(!eof(f))
	{
	  linea[i]=' ';
	  read(f,&ch,1); /* leo el retorno de carro */
	}
      else
	if (cima_pila>0) {  /* si hay ficheros en la pila, se desapilan */
	  close(f);
	  free(pila_nombres_ficheros[cima_pila]);
	  /*         linea[i]=''  seria conveniente aadir un retorno de carro */
	  /* por si el fichero del que hemos salido no termina bien            */
	  //Los ficheros KDM nunca insertan ms ficehros KDM, por tanto
	  //al extraer el fichero, seguro que el nuevo fichero no es de 
	  //nivel KDM.
	  is_kdm_level = FALSE;	  
	  cima_pila--;
	  f=pila_fich[cima_pila];
	  actualiza_contador_linea_extraccion(numlinea, cima_pila);
	} else linea[i]='$';
      
      longlineactual=i;
      /* Quitando el siguiente comentario se imprime el fichero fuente linea por */
      /* linea */
      /*     for(i=1;i<=longlineactual;i++) printf("%c",linea[i]); */
      /*     printf("\n"); */
    }
  contcarac++;
  if (obt_orig) ch = linea_orig[contcarac];
  else    ch=linea[contcarac];
  //  fprintf (stderr, "El caracter es %c\n", ch);
  return(ch);
}
 /* fin de obtch */




void obtsym()

{
  int i,j,k;
  float frac;
  float decimal;

  simbolo=nulo;
  num=0;
  esreal=FALSE;
  error_rango=FALSE;
  if(leidosigcar) leidosigcar=FALSE;
  else ch=obtch();
  
  while(simbolo==nulo) {
    switch(ch) {
    case ' ' : 
      ch=obtch(); 
      break;
    case '#' :
      obtsym(); //Averigua de qu directiva se trata
      switch(simbolo) {
      case importsim:
	obtsym();  //Averigua que subdirectiva es
	switch (simbolo) {
	case kdmsim:
	  
	  ch = obtch();
	  leidosigcar = TRUE;
	  //Puede que existan comentarios que tengamos que saltarnos...
	  while (ch == ';' || ch == '{') {
	    obtsym();
	    leidosigcar = TRUE;
	  }
	  //Ahora si que podemos saber si se ha introducido un fichero...
	  if (ch == '"') {
	    //Hay que leer todo es smbolo pues se trata del 
	    //nombre del fichero.
	    obtsym();
	    if (simbolo == cadenasim) {
	      is_kdm_level = !incluir_fichero_arlips(cadena_comillas);
	    }
	  }
	  else {
	    //No se ha especificado nombre de fichero
	    //Se toma el valor por defecto
	    is_kdm_level = !incluir_fichero_arlips(f_kdm);
	  }
	  artis_mode = TRUE;
	  nomain = TRUE; //El modo artis implica que no haya main
	  break;
	  
	default:
	  menerror_1(102);
	  break;
	} /* Fin del case de subdirectiva para el "import" */
	break;
      case includesim:
	obtsym();
	if (simbolo == cadenasim) {
	  incluir_fichero_arlips(cadena_comillas);
	}
	else menerror_1(116);
	break;
      case extfilesim:
	obtsym();
	if (simbolo == cadenasim) {
	  /* procesamos la directiva #extern*/
	  /* aadimos el fichero externo a la lista */
	  if (num_fich_ext<max_f_ext) {
	    fich_ext[num_fich_ext]=(char *)malloc(strlen(cadena_comillas) + 1);
	    strcpy(fich_ext[num_fich_ext], cadena_comillas);   
	    num_fich_ext++;
	  } else menerror_1(114);
	}
	else menerror_1 (116);
	break;
      default:
	menerror_1(102);
      } //Fin del switch del tipo de directiva
      
      //En cualquier caso, cuando se obtiene una directiva,
      //y dado que no la hemos tratado por sinsem,
      //sinsem no esperar el smbolo que obtengamos.
      //Para evitar que se produzcan errores, actualizamos 
      //el ltimo smbolo obtenido a nulo, de forma que 
      //garantizamos que se produzca una nueva iteracin del 
      //bucle del obtsym(), obteniendo un smbolo esperado.
      simbolo = nulo;



      
    case '\t': ch=obtch(); break;
    case '{': 
      do
	ch=obtch(); 
      while(ch!='}');
      ch=obtch();
      break;
      
    case ':': ch=obtch();
      if(ch=='=') simbolo=asigna;
      else {
	simbolo=dospunto;
	leidosigcar=TRUE;
      }
      break;
      
    case '.': if(enteroseguidodospuntos)
      {
	simbolo=puntopunto;
	enteroseguidodospuntos=FALSE;
      }
    else
      {
	ch=obtch();
	if(ch=='.') simbolo=puntopunto;
	else
	  {
	    simbolo=punto;
	    leidosigcar=TRUE;
	  }
      }
      break;
      
    case '<': ch=obtch();
      if(ch=='>') simbolo=distinto;
      else if(ch=='=') simbolo=menorig;
      else { simbolo=menor; leidosigcar=TRUE; }
      break;
      
    case '>': ch=obtch();
      if(ch=='=') simbolo=mayorig;
      else { simbolo=mayor; leidosigcar=TRUE; }
      break;
      
    case '=': ch=obtch();
      if(ch=='>') simbolo=implisim;
      else { simbolo=igual; leidosigcar=TRUE; }
      break;
      
    case '?': simbolo=existe; break;
      
    case '@': simbolo=paratodo; break;
      
    case '$': simbolo=dolar; break;
      
    case '+': simbolo=mas; break;
      
    case '*': simbolo=opmult; break;
      
    case '(': simbolo=abrepar; break;
      
    case ';': /* comentarios: se lee la siguiente linea */
      /* y se descarta el resto de la actual    */
      contcarac=longlineactual;
      ch=obtch();
      break;
      
    case '-': simbolo=menos; break;
      
    case '/': /* Comentario o division */
      ch=obtch();
      if (ch=='/'){
	contcarac=longlineactual;
	ch=obtch(); }
      else {
	simbolo = opdiv ;
	leidosigcar=TRUE;}
      break;
      
    case ',': simbolo=coma; break;
      
    case ')': simbolo=cierrapar; break;
      
    case '[': simbolo=abrecor; break;
      
    case ']': simbolo=cierracor; break;
      
    case '"': 
      for (k=0; k<max_cadena_comillas; k++) {
	ch = obtch();
	if (ch=='"'){
	  cadena_comillas[k]='\0';
	  simbolo = cadenasim;
	  break;
	}
	else {
	  cadena_comillas[k] = ch;
	}
      }
      if (ch != '"') {
	//El bucle acab por tamao mximo.
	menerror_1(122);
      }
      break;
      
    default:
      //Creo que no habra que hacer caso a obt_orig, pues vamos a obtener siempre
      //la cadena original
      if(/*!obt_orig &&*/   ((('A'<=ch)&&(ch<='Z')) || (('a'<=ch)&&(ch<='z')))) {
	/* es un identificador o palabra reservada */
	k=-1;
	do{
	  k++;
	  if(k<long_ident) {
	    id[k]=ch;  //En id guardo lo que el usuario haya escrito
	    if (('a'<=ch) && (ch<='z')) { //Pero idaux lo paso a maysculas
	      idaux[k]=ch-32;
	    }
	    else {
	      idaux[k] = ch;
	    }
	  }
	  ch=obtch();
	}while((('A'<=ch)&&(ch<='Z')) || (('a'<=ch)&&(ch<='z')) || (('0'<=ch)&&(ch<='9')) || (ch=='_'));
	leidosigcar=TRUE;
	//Ahora Arlips ya distingue entre maysculas y minsculas,
	//pero para hacer la bsqueda del identificador, como permitimos
	//que el usuario los escriba como quiera, en idaux hemos almacenado 
	//la cadena en maysculas, y haremos la bsqueda binaria sobre 
	//maysclas
	
	id[k+1]='\0';
	idaux[k+1] = '\0';
		  
	/* busqueda binaria de palabra reservada */
	i=1; j=num_pal_reser+1;
	do {
	  k=(int)((i+j)/2);
	  if(strcmp(idaux,pr[k])<=0) j=k-1;
	  if(strcmp(idaux,pr[k])>=0) i=k+1;
	} while(i<=j);
	//Para evitar posibles problemas, si se trataba de un identificador,
	//paso idaux a la misma cadena que el usuario ha escrito.
	//Pero si es una palabra reservada, paso id a maysculas, como se haca antes. 
	if(i-1>j) {
	  simbolo=simpalres[k];
	  strcpy(id, idaux);
	}
	else {
	  simbolo=ident;
	  strcpy(idaux, id);
	}
      }
      else if(('0'<=ch)&&(ch<='9')) {
	simbolo=numero;
	do{
	  /* almacenamos el numero como entero */
	  num=10*num+(ch-'0');
	  ch=obtch();
	}while(('0'<=ch)&&(ch<='9'));
	if(ch=='.') {
	  ch=obtch();
	  if(ch=='.') enteroseguidodospuntos=TRUE;
	  else {
	    esreal=TRUE;
	    frac=1;
	    decimal=0;
	    while(('0'<=ch)&&(ch<='9')) {
	      decimal=decimal*10+(ch-'0');
	      frac=frac*10;
	      ch=obtch();
	    }
	    num=num+decimal/frac;
	  }
	}
	else { 
	  if(num>MAXINT) {
	    error_rango=TRUE;
	    menerror_1(2);
	  }
	}
	leidosigcar=TRUE;
      }
/*       else */
/* 	//Este cdigo ya no debiera ejecutarse nunca... */
/* 	if(obt_orig && ((('a'<=ch) &&(ch<='z'))||(('a'<=ch) &&(ch<='z')))) { */
/* 	  fprintf(stderr, "Atencin, se est ejecutando cdigo no esperado referente a obt_orig\n"); */
/* 	  /\* es un identificador *\/ */
/* 	  k=-1; */
/* 	  do{ */
/* 	    k++; */
/* 	    if(k<long_ident) { */
/* 	      id[k]=ch; */
/* 	      idaux[k]=ch; */
/* 	    } */
/* 	    ch=obtch(); */
/* 	  } while((('a'<=ch) &&(ch<='z')) || (('A'<=ch)&&(ch<='Z'))||(('0'<=ch)&&(ch<='9')) || (ch=='_')); */
/* 	  leidosigcar=TRUE; */
/* 	  if(k>long_ident) menerror_1(1); */
/* 	  if(k<long_ident) { */
/* 	    id[k+1]='\0'; */
/* 	    for(i=k+1; i<long_ident; i++) idaux[i]=' '; */
/* 	    idaux[long_ident]='\0'; */
/* 	  } */
/* 	  simbolo=ident; */
/* 	} */
	else {
	  menerror_1(4);
	  fprintf(stderr, "El caracter invlido es el %c\n", ch);
	  ch=obtch();
	}
    }
  }
}
  
void empareja(simb,nerror,error_)
char simb;
int nerror;
char * error_;

{

  //errorahora = FALSE;
  if(simbolo==simb)
    {
      obtsym();
      errorahora=FALSE;
    }
  else if(!errorahora)
    {
      menerror(nerror,error_); /* evita una cascada de errores por */
      errorahora=TRUE;          /* emparejamientos erroneos */
    }
}
 
void sincroniza(simvalidos,ner,siguientes)
char * simvalidos;
int ner;
char * siguientes;
{
 char error_=FALSE;

 if(!pertenece(simbolo,simvalidos))
   if(pertenece(nulo,simvalidos))
     if(!pertenece(simbolo,siguientes))
       {
	if(simbolo!=dolar)
	  menerror(ner,&error_);
	conjunion(simvalidos,siguientes,uni);
	while((!pertenece(simbolo,uni)) && (simbolo != dolar)) obtsym();
        printf("Found simbol member of the FOLLOW() set\n");

       }
     else{}
   else
    {
     menerror(ner,&error_);
     conjunion(simvalidos,siguientes,uni);
     while((!pertenece(simbolo,uni)) && (simbolo != dolar)) obtsym();
     }
 else {}

}

int incluir_fichero_arlips(char *fich_aux) {
  //Incluye un nuevo fichero arlips para su traduccin
  //Devuelve 0 si todo va bien, o el nmero de error en caso contrario

  //Se comprueba que haya sitio en la pila de ficheros abiertos
  if (cima_pila<max_pila_f) {
    f=abrir_fichero_lectura(fich_aux);
    if (f!=-1) {
      //El fichero se abre correctamente. Hay que apilarlo.    
      actualiza_contador_linea_insercion(numlinea, cima_pila);
      cima_pila++;      
      pila_fich[cima_pila]=f;
      pila_nombres_ficheros[cima_pila] = (char *)malloc(strlen(fich_aux) + 1);
      strcpy(pila_nombres_ficheros[cima_pila], fich_aux);
      return 0;
    }
    else {
      //El fichero no se ha podido abrir...
      //Hay que restaurar el fichero antiguo para continuar
      f = pila_fich[cima_pila];
      return 121;
    }
  }
  else {
    //Si la pila estaba llena... 
    menerror_1(120);
    return 120;
  } 
}
