/*

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                Arlips compiler v. 3.1           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

File kdm2arlips_generator.c, parses kdm definition to arlips code
April 2006

Copyright (C) 2006 Emilio Vivancos Rubio

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                              Contact                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Author:              Emilio Vivancos Rubio  vivancos@dsic.upv.es
Developer Arlips v2: Mario Gonzlez Gimnez mgonzale@dsic.upv.es
Developer Arlips v3: Carlos Garca Montoro  cargarcia@dsic.upv.es

Departamento de Sistemas Informticos y Computacin
Universidad Politcnica de Valencia
Camino de Vera s/n
46022 Valencia
Spain

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

*/

/*!
  \file kdm2arlips_generator.c
  \date Time-stamp: <30-11-2005 17:10>
  \author Carlos Garca Montoro
  \based on artis_kdm_parser by M. Rebollo
  \version 0.3

  \brief kdm to Arlips specific parsing functions


*/


  
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "../../../kdm/include/kdm.h"
#include "kdm2arlips_generator.h"
#include "publicsymbol.h"

#define HEADERFILE_ARL "kdm.arl"

//Global variables definition
FILE *fheader_arl;                  //!< output file

/*! 
  \fn void hfileheading( const char *xmlfile )
  \brief begins the header user file
  \param xmlfile the xml file to parse
  \exception userclass.h header file does not exist
*/
static void 
//-----------------------------------------------
hfileheading( const char *xmlfile )
//-----------------------------------------------
{
  time_t today;

  //gets the current data and generates header for the file
  today = time( NULL );
  fprintf( fheader_arl, "{==========================================\n" );
  fprintf( fheader_arl, "      file: kdm.arl                  \n");
  fprintf( fheader_arl, "      generated from %s                    \n", xmlfile );
  fprintf( fheader_arl, "      date: %s                             \n", ctime( &today ) );
  fprintf( fheader_arl, "      PLEASE, DO NOT MODIFIY THIS FILE     \n" );
  fprintf( fheader_arl, " ==========================================}\n\n" );
} //hfileheading()



void 
//-----------------------------------------------
arl_startclass( const char *classname )
//-----------------------------------------------
{
  fprintf( fheader_arl, "(defclass %s\n", classname );
} //arl_startclass()


void 
//-----------------------------------------------
arl_endclass()
//-----------------------------------------------
{
  fprintf( fheader_arl, ")\n\n" );
} //arl_endclass()
//CGM: A class could be empty when all its types are not 
//supported by Arlips. 


void 
//-----------------------------------------------
arl_herencia( const char *c_hija, 
	      const char *c_padre )
//-----------------------------------------------
{
  fprintf(f_header, " isa %s\n", c_padre);
}


void 
//-----------------------------------------------
arl_addslot( const char *classname,
	     const char *slotname  )
//-----------------------------------------------
{
  int valid_type = 0;
  char arl_type [12];

  //Getting the type of the slot 
  switch (pub_getslottype_fromclass(classname, slotname)) {
  case KDM_UINT:
  case KDM_LONG:
  case KDM_ULONG:
  case KDM_INT64:
  case KDM_UINT64:
    fprintf(stderr, "Warning: Arlips supports only the integer \"int\" declaration.\n");
    fprintf(stderr, "   The parser will cast the slot %s of class %s to int\n\n", slotname, classname);
  case KDM_INT:
    strcpy( arl_type,  "integer");
    valid_type = 1;
    break;

  case KDM_DOUBLE:
    fprintf(stderr, "Warning: Arlips supports only real \"float\" declaration.\n");
    fprintf(stderr, "   The parser will cast the slot %s of class %s to float\n\n", slotname, classname);
  case KDM_FLOAT:
    strcpy( arl_type, "real");
    valid_type = 1;
    break;

  case KDM_ARRAY:
    //Los arrays se declaran mediante arl_addarray.
    //Por tanto, aqu no deben producir un error, 
    //pero tampoco deben declararse. Es decir, aqu no deben hacer nada
    break;

  default:
    fprintf( stderr, "Warning: Arlips does not support the type of the slot %s of class %s\n", slotname, classname);
    fprintf( stderr, "   The slot %s of class %s is not being translated\n", slotname, classname);
    fprintf( stderr, "   The slot will be ignored in every instance\n\n");
    break;
  }

  //if the type is accepted by Arlips (and is not an array), print it
  if (valid_type) {
    fprintf( fheader_arl, "   (slot %s (type %s))\n", slotname, arl_type);
  }

} //arl_addslot()


void
//----------------------------------------------- 
arl_start_instances_declaration()
//-----------------------------------------------
{
  fprintf( fheader_arl, "\n(definstances kdm_instances\n");  
} //arl_starinstances


void
//-----------------------------------------------
arl_end_instances_declaration()
//-----------------------------------------------
{
  fprintf( fheader_arl, ")\n\n");  
} //arl_endinstances
  


void 
//-----------------------------------------------
arl_addslotvalue( const char *classname,
		 const char *instname,
		  const char *slotname,
		  const char *slotvalue)
//-----------------------------------------------
{
  int i, size_array;
  char *token_arl;
  kdm_t hola;

  //Falta obtener el tipo del slot.
  //Falta por obtener el subtipo en caso de arrays.
  //Falta por obtener el tamao del array en caso de array.

  //sets the defalut value for the slot


  switch (pub_getslottype_fromclass(classname, slotname)) {
  case KDM_UINT:
  case KDM_LONG:
  case KDM_ULONG:
  case KDM_INT64:
  case KDM_UINT64:
  case KDM_DOUBLE:
    //Todos estos son tipos no nativos arlips, pero arlips los tratar como nativos
    //con la precisin de un int o un float. La advertencia se imprimi al declarar el slot...
  case KDM_INT:
  case KDM_FLOAT:
    fprintf( fheader_arl, "      (%s %s)\n", slotname, slotvalue );
    break;

  case KDM_ARRAY:
    switch (pub_getarrayelemtype_fromclass(classname, slotname)) {
    case KDM_UINT:
    case KDM_LONG:
    case KDM_ULONG:
    case KDM_INT64:
    case KDM_UINT64:
    case KDM_DOUBLE:
      //Tipos a los que se les aplic el cast.
    case KDM_INT:
    case KDM_FLOAT:
      fprintf(fheader_arl, "      (%s (", slotname);
      token_arl=(char*)strtok(slotvalue, " "); 
      size_array = pub_getarraysize_fromclass(classname, slotname);
      //printf("el tamao del array es:%d\n",size_array);
      for( i = 0; i < size_array && token_arl!=NULL; i++ ){//!!
	fprintf(fheader_arl, " %s ", token_arl);
	token_arl=(char*)strtok( NULL, " " );
      } //for each default value for the array
      fprintf(fheader_arl, "))\n");
      break;
    default:
      //El array es de un tipo no soportado por Arlips.
      //El aviso habr sido impreso por arl_addarray
      break;
    } //switch type of the array
    break;

  default:
    //El tipo del valor por defecto no es soportado por Arlips. Se omite.
    //El mensaje de error fue impreso por arl_addslot
    break;
  }
} //arl_addslotvalue()


void 
//-----------------------------------------------
arl_addarray( const char *classname,
	      const char *slotname  )
//-----------------------------------------------
{
  int valid_type = 0;
  char arl_type[12];
  
  switch (pub_getarrayelemtype_fromclass(classname, slotname)) {
  case KDM_UINT:
  case KDM_LONG:
  case KDM_ULONG:
  case KDM_INT64:
  case KDM_UINT64:
    fprintf(stderr, "Warning: Arlips supports only the integer \"int\" declaration.\n");
    fprintf(stderr, "   The parser will cast the slot %s of class %s to an array of int\n\n", slotname, classname);
  case KDM_INT:
    strcpy(arl_type, "integer");
    valid_type = 1;
    break;

  case KDM_DOUBLE:
    fprintf(stderr, "Warning: Arlips supports only the real \"float\" declaration.\n");
    fprintf(stderr, "   The parser will cast the slot %s of class %s to an array of float\n\n", slotname, classname);
  case KDM_FLOAT:
    strcpy(arl_type, "real");
    valid_type = 1;
    break;
  
  default:
    fprintf( stderr, "Warning: Arlips does not support the type of the slot %s of class\n", slotname, classname);
    fprintf( stderr, "   The slot %s of class %s is not being translated\n", slotname, classname);
    fprintf( stderr, "   The slot will be ignored in every instance\n\n");
    break;
  }

  if (valid_type) {
    fprintf(fheader_arl, "   (slot %s (type array[%d] of %s))\n", 
	    slotname, 
	    pub_getarraysize_fromclass(classname, slotname),
	    arl_type);
  }

} //arl_addarray()


void 
//-----------------------------------------------
arl_addinstance( const char *classname, 
		 const char *instname )
//-----------------------------------------------
{

  fprintf( fheader_arl, "   (%s of %s\n", 
	   instname, classname );

} //arl_addinstance()


void
//----------------------------------------------- 
arl_end_addinstance()
//-----------------------------------------------
{
  fprintf( fheader_arl, "   )\n");
} //arl_end_addinstance



int
//-----------------------------------------------
arl_beginparsing( const char *xmlfile )
//-----------------------------------------------
{

  // create header file
  if( (fheader_arl = fopen( HEADERFILE_ARL, "w" )) == NULL ){
    fprintf( stderr, "Error creating %s file\n", HEADERFILE_ARL );
    exit( -1 );
  }
  hfileheading( xmlfile );

} //arl_beginparsing()


void
//-----------------------------------------------
arl_endparsing( const char *xmlfile )
//-----------------------------------------------
{
  fclose( fheader_arl );

} //arl_endparsing()
