/*

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                Arlips compiler v. 3.1           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

File kdm2arlips_generator.h, parses kdm definition to arlips code
April 2006

Copyright (C) 2006 Emilio Vivancos Rubio

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                              Contact                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Author:              Emilio Vivancos Rubio  vivancos@dsic.upv.es
Developer Arlips v2: Mario Gonzlez Gimnez mgonzale@dsic.upv.es
Developer Arlips v3: Carlos Garca Montoro  cargarcia@dsic.upv.es

Departamento de Sistemas Informticos y Computacin
Universidad Politcnica de Valencia
Camino de Vera s/n
46022 Valencia
Spain

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

*/


/*!
  \file kdm2arlips_generator.h
  \date Time-stamp: <01-12-2005>
  \author Carlos Garca Montoro
  \based on artis_kdm_parser by M. Rebollo
  \version 0.3
  
  \brief kdm specific parsing functions
*/


/*!
  \fn int arl_beginparsing( const char *xmlfile )
  \brief Inits the needed files before begin parsing
*/
int arl_beginparsing( const char *xmlfile );
//NOTA: Replantearse el uso del nombre del fichero xml

/*!
  \fn void arl_endparsing( const char *xmlfile )
  \brief Closes the used files after finishing parsing
*/
void arl_endparsing( const char *xmlfile );
//NOTA: Si el fichero generado siempre se va a llamar igual
//entonces el parmetro no es necesario.



/*!
  \fn void arl_startclass( const char *classname )
  \brief Called when an opening label for the class is readed.
*/
void arl_startclass( const char *classname );

/*!
  \fn void arl_endclass( const char *classname )
  \brief Called when closing label for the class is readed.
*/
void arl_endclass();


/*!
  \fn void arl_herencia( const char *c_hija, const char *c_padre )
  \brief Called when class c_hija inherits form class c_padre
*/
void arl_herencia( const char *c_hija,
		   const char *c_padre );

/*!
  \fn void arl_addslot()
  \brief Called to add a new slot to the user files.
*/
void arl_addslot( const char *classname, 
		  const char *slotname );

/*!
  \fn void arl_start_instances_declaration()
  \brief Called to start the declaration of instances
*/
void arl_start_instances_declaration();


/*!
  \fn void arl_end_instances_declaration()
  \brief Called to end the declaration of instances
*/
void arl_end_instances_declaration();


/*!
  \fn void arl_addslotvalue()
  \brief Called to add a new value for an atomic slot.
*/
void arl_addslotvalue(const char *classname, 
		      const char *instname, 
		      const char *slotname, 
		      const char *slotvalue );

/*!
  \fn void arl_addarray()
  \brief Called to add an array-typed slot.
*/
void arl_addarray( const char *classname,
		   const char *slotname);
/*!
  \fn void arl_addinstance( const char *classname, const char *instname )
  \brief Called to add a new instance to the user files.
*/
void arl_addinstance( const char *classname, 
		      const char *instname );

/*!
  \fn void arl_end_addinstance()
  \brief Called to end the declararion of an instance after the default values
*/
void arl_end_addinstance();
