#include "tipos_.h"
#include <stdio.h>


typedef enum {head, tail, pophead, poptail, addhead, addtail} funcion_lista;


void print_array(FILE *fd, void *array, const enum_tipos tipo, const int tam) {
  int i;
  int *int_array;
  float *float_array;

  fprintf(fd, "(");

  switch (tipo) {
  case boolean:
  case entero:
    int_array = (int *)array;
    for (i = 0; i < tam; i++) {
      fprintf(fd, " %d ", int_array[i]);
    }
    break;
  case real:
    float_array = (float *)array;
    for (i = 0; i < tam; i++) {
      fprintf(fd, " %f ", float_array[i]);
    }
    break;
  }
  fprintf(fd, ")");
}


void error_lista(const funcion_lista f, const char *l) {
  char *msg;

  switch (f) {
  case head:
    msg = "Arlips cannot execute head on the empty list";
    break;

  case tail:
    msg = "Arlips cannot execute tail on the empty list";
    break;

  case pophead:
  case poptail:
    msg = "Arlips cannot pop an element from the empty list";
    break;

  case addhead:
  case addtail:
    msg = "Arlips cannot add an element on the full list";
    break;
  } //del switch (f)

  fprintf(stderr, "ERROR: %s %s\n", msg, l);
} //de error_lista()


void inicializa_lent(LISTA_GENERICA_ENTEROS *lent, const int tamlis, char *nombre, int *comienzo) {
	lent->nombre = nombre;
	lent->cabeza=-1;
	lent->cola =0;
	lent->tam = tamlis;
	lent->num_elem = 0;
	lent->lista = comienzo;
}

void inicializa_lreal(LISTA_GENERICA_REALES *lreal, const int tamlis, char *nombre, float *comienzo) {
	lreal->nombre = nombre;
	lreal->cabeza=-1;
	lreal->cola =0;
	lreal->tam = tamlis;
	lreal->num_elem = 0;
	lreal->lista = comienzo;
}

void print_lent(FILE *fd, const LISTA_GENERICA_ENTEROS *l) {
  int i;
  fprintf(fd, "(");
  for(i = 0; i < l->num_elem; i++) {
    fprintf(fd, " %d ", (*l).lista[((*l).cabeza+i)%(*l).tam]);
  }
  fprintf(fd, ")");
}


void print_lreal(FILE *fd, const LISTA_GENERICA_REALES *l) {
  int i;
  fprintf(fd, "(");
  for(i = 0; i < l->num_elem; i++) {
    fprintf(fd, " %f ", (*l).lista[((*l).cabeza+i)%(*l).tam]);
  }
  fprintf(fd, ")");
}


int head_lent(const LISTA_GENERICA_ENTEROS *lent) {
	int devol;
	int cab;

	if ((*lent).cabeza==-1) {   /* LISTA VACIA */
		//lista_vacia=TRUE;
		error_lista(head, (*lent).nombre);
		return(0);
	}
	devol = (*lent).lista[(*lent).cabeza];
	return (devol);
}

int tail_lent(const LISTA_GENERICA_ENTEROS *lent) {
	int devol;
	int col;

	if ((*lent).cabeza==-1) {   /* LISTA VACIA */
		//lista_vacia=TRUE;
		error_lista(tail, (*lent).nombre);
		return(0);
	}
	col= (*lent).cola;
	if (col==0) devol = (*lent).lista[(*lent).tam-1];
	 else devol = (*lent).lista[(*lent).cola-1];

	return (devol);
}

int pop_head_lent(LISTA_GENERICA_ENTEROS *lent) {
	int devol;
	int cab;

	if ((*lent).cabeza==-1) {   /* LISTA VACIA */
		//lista_vacia=TRUE;
		error_lista(pophead, (*lent).nombre);
		return(0);
	}
	devol = (*lent).lista[(*lent).cabeza];
	cab= (*lent).cabeza;
	if ((*lent).cabeza==(*lent).tam-1) (*lent).cabeza=0;
		else   (*lent).cabeza++;

	if ((*lent).cabeza==(*lent).cola) {
		//lista_vacia = TRUE;
		(*lent).cabeza=-1;
		(*lent).cola=0;
	}
	(*lent).num_elem--;
	return (devol);
}

int pop_tail_lent(LISTA_GENERICA_ENTEROS *lent) {
	int devol;
	int col;

	if ((*lent).cabeza==-1) {   /* LISTA VACIA */
		//lista_vacia=TRUE;
		error_lista(poptail, (*lent).nombre);
		return(0);
	}
	col= (*lent).cola;
	if (col==0) devol = (*lent).lista[(*lent).tam-1];
	 else devol = (*lent).lista[(*lent).cola-1];

	if ((*lent).cola==0) (*lent).cola=(*lent).tam-1;
	else   (*lent).cola--;

	if ((*lent).cabeza==(*lent).cola) {
		//lista_vacia = TRUE;
		(*lent).cabeza=-1;
		(*lent).cola=0;
	}
	(*lent).num_elem--;
	return (devol);
}

void add_head_lent(const int num, LISTA_GENERICA_ENTEROS *lent) {
	int cab;

	if ((*lent).cabeza==(*lent).cola) {    /* si esta llena */
		//lista_llena=TRUE;
		error_lista(addhead, (*lent).nombre);
		return;
	}

	cab = (*lent).cabeza;

	if ((*lent).cabeza<=0) (*lent).cabeza=(*lent).tam-1;
		else  (*lent).cabeza--;

	(*lent).lista[(*lent).cabeza] = num;
	(*lent).num_elem++;
}

void add_tail_lent(const int num, LISTA_GENERICA_ENTEROS *lent) {
	int col;

	if ((*lent).cabeza==(*lent).cola) {
		//lista_llena=TRUE;
		error_lista(addtail, (*lent).nombre);
		return;
	}

	if ((*lent).cabeza==-1) /* lista vacia */
	{
		(*lent).cabeza = (*lent).tam-1;
		(*lent).lista[(*lent).cabeza]=num;
	} else {
		col= (*lent).cola;
		if ((*lent).cola==(*lent).tam-1)  (*lent).cola=0;
			else (*lent).cola++;
		(*lent).lista[col] = num;
	}
	(*lent).num_elem++;
}

float head_lreal(const LISTA_GENERICA_REALES *lreal) {
	float devol;
	int cab;

	if ((*lreal).cabeza==-1) {   /* LISTA VACIA */
		//lista_vacia=TRUE;
		error_lista(head, (*lreal).nombre);
		return(0.0);
	}
	devol = (*lreal).lista[(*lreal).cabeza];
	return (devol);
}

float tail_lreal(const LISTA_GENERICA_REALES *lreal) {
	float devol;
	int col;

	if ((*lreal).cabeza==-1) {   /* LISTA VACIA */
		//lista_vacia=TRUE;
		error_lista(tail, (*lreal).nombre);
		return(0.0);
	}
	col= (*lreal).cola;
	if (col==0) devol = (*lreal).lista[(*lreal).tam-1];
	       else devol = (*lreal).lista[(*lreal).cola-1];

	return (devol);
}

float pop_head_lreal(LISTA_GENERICA_REALES *lreal) {
	float devol;
	int cab;

	if ((*lreal).cabeza==-1) {   /* LISTA VACIA */
		//lista_vacia=TRUE;
		error_lista(pophead, (*lreal).nombre);
		return(0.0);
	}
	devol = (*lreal).lista[(*lreal).cabeza];
	cab= (*lreal).cabeza;
	if ((*lreal).cabeza==(*lreal).tam-1) (*lreal).cabeza=0;
		else   (*lreal).cabeza++;

	if ((*lreal).cabeza==(*lreal).cola) {
		//lista_vacia = TRUE;
		(*lreal).cabeza=-1;
		(*lreal).cola=0;
	}
	(*lreal).num_elem--;
	return (devol);
}

float pop_tail_lreal(LISTA_GENERICA_REALES *lreal) {
	float devol;
	int col;

	if ((*lreal).cabeza==-1) {   /* LISTA VACIA */
		//lista_vacia=TRUE;
		error_lista(poptail, (*lreal).nombre);
		return(0.0);
	}
	col= (*lreal).cola;
	if (col==0) devol = (*lreal).lista[(*lreal).tam-1];
	       else devol = (*lreal).lista[(*lreal).cola-1];

	if ((*lreal).cola==0) (*lreal).cola=(*lreal).tam-1;
	else   (*lreal).cola--;

	if ((*lreal).cabeza==(*lreal).cola) {
		//lista_vacia = TRUE;
		(*lreal).cabeza=-1;
		(*lreal).cola=0;
	}
	(*lreal).num_elem--;
	return (devol);
}

void add_head_lreal(const float num, LISTA_GENERICA_REALES *lreal) {
	int cab;

	if ((*lreal).cabeza==(*lreal).cola) {    /* si esta llena */
		//lista_llena=TRUE;
		error_lista(addhead, (*lreal).nombre);
		return;
	}

	cab = (*lreal).cabeza;

	if ((*lreal).cabeza<=0) (*lreal).cabeza=(*lreal).tam-1;
		else  (*lreal).cabeza--;

	(*lreal).lista[(*lreal).cabeza] = num;
	(*lreal).num_elem++;
}

void add_tail_lreal(const float num, LISTA_GENERICA_REALES *lreal) {
	int col;

	if ((*lreal).cabeza==(*lreal).cola) {
		//lista_llena=TRUE;
		error_lista(addtail, (*lreal).nombre);
		return;
	}

	if ((*lreal).cabeza==-1) /* lista vacia */
	{
		(*lreal).cabeza = (*lreal).tam-1;
		(*lreal).lista[(*lreal).cabeza]=num;
	} else {
		col= (*lreal).cola;
		if ((*lreal).cola==(*lreal).tam-1)  (*lreal).cola=0;
			else (*lreal).cola++;
		(*lreal).lista[col] = num;
	}
	(*lreal).num_elem++;
}
int size_lent(const LISTA_GENERICA_ENTEROS *lent) {
	return (lent->num_elem);
}

int size_lreal(const LISTA_GENERICA_REALES *lreal) {
	return (lreal->num_elem);
}


